@@ -5,6 +5,7 @@ import java.util.HashMap;
 import java.util.List;
 
 import org.gcube.portlets.user.gisviewer.client.Constants;
+import org.gcube.portlets.user.gisviewer.client.GisViewerPanel;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.GisViewerBaseLayerInterface;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItem;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.MapViewInfo;
@@ -13,6 +14,7 @@ import org.gwtopenmaps.openlayers.client.LonLat;
 import org.gwtopenmaps.openlayers.client.Map;
 import org.gwtopenmaps.openlayers.client.MapOptions;
 import org.gwtopenmaps.openlayers.client.MapWidget;
+import org.gwtopenmaps.openlayers.client.ZIndexBase;
 import org.gwtopenmaps.openlayers.client.control.Control;
 import org.gwtopenmaps.openlayers.client.control.DrawFeature;
 import org.gwtopenmaps.openlayers.client.control.DrawFeature.FeatureAddedListener;
@@ -42,6 +44,7 @@ import org.gwtopenmaps.openlayers.client.util.JSObject;
 
 import com.extjs.gxt.ui.client.widget.ContentPanel;
 import com.extjs.gxt.ui.client.widget.layout.FitLayout;
+import com.google.gwt.core.shared.GWT;
 import com.google.gwt.user.client.Element;
 import com.google.gwt.user.client.Window;
 import com.google.gwt.user.client.ui.DockPanel;
@@ -49,7 +52,7 @@ import com.google.gwt.user.client.ui.Widget;
 
 public class OpenLayersMap {
 
-	protected static final int MAX_ZINDEX = 100000;
+	protected static final int MAX_ZINDEX = 280;
 	private List<LayerItem> layerItems = new ArrayList<LayerItem>();
 	private Map map;
 	private MapWidget mapWidget;
@@ -81,6 +84,37 @@ public class OpenLayersMap {
 	private ToolBarPanel toolBarPanel;
 	private List<GisViewerBaseLayerInterface> baseLayersIntefaces;
 	
+	//ADDED BY FRANCESCO M. TO FIX # support 832
+	public class ZIndexBaseGisViewer extends ZIndexBase{
+		
+		protected ZIndexBaseGisViewer(JSObject jsObject) {
+			super(jsObject);
+		}
+
+		//Ideally this is initialized from OpenLayers, but it seems that
+		// would require instantiating OpenLayers.Map to copy default Z_INDEX_BASE
+		public ZIndexBaseGisViewer(){
+			this(JSObject.createJSObject());
+			setForBaseLayers(20);
+			setForOverlays(80);
+			setForFeatures(140);
+			setForPopups(200);
+			setForControls(240);
+		}
+
+		public ZIndexBaseGisViewer(int baseLayerZIndexBase,
+				int overlayZIndexBase,
+				int featureZIndexBase,
+				int popupZIndexBase,
+				int controlZIndexBase){
+			this(JSObject.createJSObject());
+			setForBaseLayers(baseLayerZIndexBase);
+			setForOverlays(overlayZIndexBase);
+			setForFeatures(featureZIndexBase);
+			setForPopups(popupZIndexBase);
+			setForControls(controlZIndexBase);
+		}
+	}
 	// constructor
 	public OpenLayersMap(String w, String h, int numZoomLevels, // projection, transectUrl removed
 			String title, final OpenLayersHandler handler) {
@@ -94,12 +128,16 @@ public class OpenLayersMap {
 		mapOptions.setControls(new JObjectArray(new JSObject[] {}));
 		mapOptions.setNumZoomLevels(numZoomLevels);
 		mapOptions.setProjection(defaultProjection);
+		
+		mapOptions.setZIndexBase(new ZIndexBaseGisViewer());
+		
 
 		// let's create map widget and map objects
 		mapWidget = new MapWidget(w, h, mapOptions); // map widget
 		map = mapWidget.getMap(); // map js-mapped object		
 		mapWidget.setTitle(title); // set title
 		map.setMaxExtent(defaultBounds);
+
 		//map.getJSObject().setProperty("allOverlays", true);
 
 		// set click listener on the map
@@ -155,7 +193,32 @@ public class OpenLayersMap {
 		
 //		addBaseLayers();
 
-		addBaseLayerToOpenLayerMap(Constants.TRUE_MARBLE_TITLE, Constants.TRUE_MARBLE_NAME, Constants.TRUE_MARBLE_URL);
+		String wmsURI = GisViewerPanel.resources.baseLayer().getText();
+
+		String[] params = wmsURI.split("\\?");
+		
+		if(params.length<2){
+			GWT.log("ERROR: No base layer found!");
+		}
+		
+		String baseLayerURI = params[0];
+		String[] parameter = params[1].split("&");
+		
+		String baseLayerName = "";
+		
+		for (String string : parameter) {
+			if(string.startsWith("layers=")){
+				baseLayerName = string.substring(7);
+				break;
+			}
+		}
+		
+		if(baseLayerURI==null || baseLayerURI.isEmpty() || baseLayerName.isEmpty()){
+			GWT.log("ERROR: No base layer found!");
+		}
+			
+		
+		addBaseLayerToOpenLayerMap("True Marble", baseLayerName, baseLayerURI);
 		
 		addControl(mouseDefaults);
 		addControl(pamZoomBar);
@@ -596,14 +659,39 @@ public class OpenLayersMap {
 //		System.out.println("LAYERS NEW ORDER");
 		maxZindex = 0;
 		for (Layer layer : baseLayers) {
-			if (Integer.parseInt((String)layer.getZIndex())>maxZindex)
-				maxZindex = Integer.parseInt((String)layer.getZIndex());
+			if (Integer.parseInt((String)layer.getZIndex())>maxZindex){
+				
+				//ADDED BY FRANCESCO M.
+				int zindex = Integer.parseInt((String)layer.getZIndex());
+				if(zindex>=MAX_ZINDEX){
+					zindex-=-50;
+					GWT.log("zindex to base layer is >=MAX_ZINDEX, recalculate as: "+zindex);
+				}
+				
+				maxZindex = zindex;
+				layer.setZIndex(maxZindex);
+				GWT.log("base layer: "+layer.getName()+" have maxZindex is: "+zindex);
+				//OLD CODE
+//				maxZindex = Integer.parseInt((String)layer.getZIndex());
+			}
 		}
 
 		for (LayerItem layerItem : layerItems) {
 //			System.out.println("" + (layerItem.getOrder()+zIndexCounter) + ") "	+ layerItem.getLayer() + " (" + layerItem.getId() + ")");
 			Layer layer = mappingLayerItemsToLayers.get(layerItem);
-			layer.setZIndex(maxZindex + (int)layerItem.getOrder());
+			
+			//ADDED BY FRANCESCO M.
+			int newOrder = maxZindex + (int)layerItem.getOrder();
+			
+			if(newOrder>=MAX_ZINDEX){
+				newOrder-=(layerItems.size()-10);
+				GWT.log("newOrder>=MAX_ZINDEX is true, recalculate newOrder: "+newOrder);
+			}
+			GWT.log("layer: "+layerItem.getName()+" setting ZIndex: "+newOrder);
+			layer.setZIndex(newOrder);
+		
+			//OLD COD
+//			layer.setZIndex(maxZindex + (int)layerItem.getOrder());
 		}
 	}
 
